@@ -6,13 +6,28 @@ import asyncio
 import json
 import os
 import sys
-import uuid
+from typing import Generator, Optional
 
-import requests
+import tls_client
 import websockets.client as websockets
 
 DELIMITER = "\x1e"
 
+HEADERS = {
+    "user-agent": (
+        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
+        "Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.41"
+    ),
+    "origin": "https://www.bing.com",
+    "referer": "https://www.bing.com/",
+    "sec-ch-ua": '"Chromium";v="110", "Not A(Brand";v="24", "Microsoft Edge";v="110"',
+    "sec-ch-ua-platform": "Windows",
+}
+
+
+class NotAllowedToAccess(Exception):
+    pass
+
 
 def append_identifier(msg: dict) -> str:
     """
@@ -34,7 +49,7 @@ class ChatHubRequest:
         conversation_id: str,
         invocation_id: int = 0,
     ) -> None:
-        self.struct: dict
+        self.struct: dict = {}
 
         self.client_id: str = client_id
         self.conversation_id: str = conversation_id
@@ -93,50 +108,29 @@ class Conversation:
             "conversationSignature": None,
             "result": {"value": "Success", "message": None},
         }
-        # Build request
-        headers = {
-            "accept": "application/json",
-            "accept-encoding": "gzip, deflate",
-            "accept-language": "en-US,en;q=0.9",
-            "content-type": "application/json",
-            "sec-ch-ua": '"Microsoft Edge";v="111", "Not(A:Brand";v="8", "Chromium";v="111"',
-            "sec-ch-ua-arch": '"x86"',
-            "sec-ch-ua-bitness": '"64"',
-            "sec-ch-ua-full-version": '"111.0.1652.0"',
-            "sec-ch-ua-full-version-list": '"Microsoft Edge";v="111.0.1652.0", "Not(A:Brand";v="8.0.0.0", "Chromium";v="111.0.5551.0"',
-            "sec-ch-ua-mobile": "?0",
-            "sec-ch-ua-model": "",
-            "sec-ch-ua-platform": '"Linux"',
-            "sec-ch-ua-platform-version": '"5.19.0"',
-            "sec-fetch-dest": "empty",
-            "sec-fetch-mode": "cors",
-            "sec-fetch-site": "same-origin",
-            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.0.0",
-            "x-ms-client-request-id": str(uuid.uuid4()),
-            "x-ms-useragent": "azsdk-js-api-client-factory/1.0.0-beta.1 core-rest-pipeline/1.10.0 OS/Linuxx86_64",
-        }
-        # Create cookies
-        if os.environ.get("BING_U") is None:
-            url = "https://bing.kpham.workers.dev/"
-            cookies = {}
-        else:
-            cookies = {
-                "_U": os.environ.get("BING_U"),
-            }
-            url = "https://www.bing.com/turing/conversation/create"
+        self.session = tls_client.Session(client_identifier="chrome_108")
+        cookie_file = json.loads(
+            open(os.environ.get("COOKIE_FILE"), "r", encoding="utf-8").read()
+        )
+        for cookie in cookie_file:
+            self.session.cookies.set(cookie["name"], cookie["value"])
+        url = "https://www.bing.com/turing/conversation/create"
         # Send GET request
-        response = requests.get(
+        response = self.session.get(
             url,
-            headers=headers,
-            cookies=cookies,
-            timeout=30,
+            timeout_seconds=30,
+            headers=HEADERS,
+            allow_redirects=True,
         )
         if response.status_code != 200:
+            print(f"Status code: {response.status_code}")
+            print(response.text)
             raise Exception("Authentication failed")
-        # Return response
         try:
             self.struct = response.json()
-        except json.decoder.JSONDecodeError as exc:
+            if self.struct["result"]["value"] == "UnauthorizedRequest":
+                raise NotAllowedToAccess(self.struct["result"]["message"])
+        except (json.decoder.JSONDecodeError, NotAllowedToAccess) as exc:
             raise Exception(
                 "Authentication failed. You have not been accepted into the beta.",
             ) from exc
@@ -148,7 +142,7 @@ class ChatHub:
     """
 
     def __init__(self, conversation: Conversation) -> None:
-        self.wss: websockets.WebSocketClientProtocol = None
+        self.wss: Optional[websockets.WebSocketClientProtocol] = None
         self.request: ChatHubRequest
         self.loop: bool
         self.task: asyncio.Task
@@ -158,7 +152,7 @@ class ChatHub:
             conversation_id=conversation.struct["conversationId"],
         )
 
-    async def ask_stream(self, prompt: str) -> str:
+    async def ask_stream(self, prompt: str) -> Generator[str, None, None]:
         """
         Ask a question to the bot
         """
@@ -167,10 +161,16 @@ class ChatHub:
             if self.wss.closed:
                 self.wss = await websockets.connect(
                     "wss://sydney.bing.com/sydney/ChatHub",
+                    extra_headers=HEADERS,
+                    max_size=None,
                 )
                 await self.__initial_handshake()
         else:
-            self.wss = await websockets.connect("wss://sydney.bing.com/sydney/ChatHub")
+            self.wss = await websockets.connect(
+                "wss://sydney.bing.com/sydney/ChatHub",
+                extra_headers=HEADERS,
+                max_size=None,
+            )
             await self.__initial_handshake()
         # Construct a ChatHub request
         self.request.update(prompt=prompt)
@@ -220,7 +220,7 @@ class Chatbot:
             if final:
                 return response
 
-    async def ask_stream(self, prompt: str) -> str:
+    async def ask_stream(self, prompt: str) -> Generator[str, None, None]:
         """
         Ask a question to the bot
         """
@@ -321,8 +321,10 @@ if __name__ == "__main__":
     )
     parser = argparse.ArgumentParser()
     parser.add_argument("--no-stream", action="store_true")
-    parser.add_argument("--bing-cookie", type=str, default="")
+    parser.add_argument(
+        "--cookie-file", type=str, default="cookies.json", required=True
+    )
+    args = parser.parse_args()
+    os.environ["COOKIE_FILE"] = args.cookie_file
     args = parser.parse_args()
-    if args.bing_cookie:
-        os.environ["BING_U"] = args.bing_cookie
     asyncio.run(main())
